@@ -4,9 +4,10 @@ import pathlib
 import platform
 import sys
 from json import JSONDecodeError
-from typing import Any, Dict, List, NamedTuple, Optional, Set
+from typing import Any, Dict, List, NamedTuple, Optional, Set, Union
 
 import click
+import typer
 from globus_sdk import AuthClient, GlobusAPIError, NativeAppAuthClient
 from globus_sdk.auth.token_response import OAuthTokenResponse
 from globus_sdk.authorizers import (
@@ -52,16 +53,32 @@ class TokenSet(NamedTuple):
     dependent_scopes: Set[str]
 
 
+TokensInTokenCache = Dict[str, Union[TokenSet, Dict[str, TokenSet]]]
+
+
 class TokenCache:
     def __init__(self, token_store: str):
         self.token_store = token_store
-        self.tokens: Dict[str, TokenSet] = {}
+        self.tokens: TokensInTokenCache = {}
         self.modified = False
 
+    @property
+    def tokens_for_environment(self) -> Dict[str, TokenSet]:
+        """
+        We will sub-key the full token set for environments other than production
+        """
+        environ = os.environ.get("GLOBUS_SDK_ENVIRONMENT")
+        if environ in {None, "production", "prod"}:
+            return self.tokens
+        environ_cache_key = "__" + environ
+        if environ_cache_key not in self.tokens:
+            self.tokens[environ_cache_key]: Dict[str, TokenSet] = {}
+        return self.tokens[environ_cache_key]
+
     def set_tokens(self, scope: str, tokens: TokenSet) -> TokenSet:
-        if scope in self.tokens:
+        if scope in self.tokens_for_environment:
             dependent_scopes = set(tokens.dependent_scopes).union(
-                set(self.tokens[scope].dependent_scopes)
+                set(self.tokens_for_environment[scope].dependent_scopes)
             )
             new_token_set = TokenSet(
                 access_token=tokens.access_token,
@@ -69,9 +86,9 @@ class TokenCache:
                 expiration_time=tokens.expiration_time,
                 dependent_scopes=dependent_scopes,
             )
-            self.tokens[scope] = new_token_set
+            self.tokens_for_environment[scope] = new_token_set
         else:
-            self.tokens[scope] = tokens
+            self.tokens_for_environment[scope] = tokens
         self.modified = True
         return tokens
 
@@ -81,12 +98,23 @@ class TokenCache:
             # the tokens. If not, even if we have a token for the base scope, we
             # shouldn't return that because it won't work for the new scope.
             base_scope = scope.split("[")[0]
-            tokens = self.tokens.get(base_scope)
+            tokens = self.tokens_for_environment.get(base_scope)
             if not tokens or scope not in getattr(tokens, "dependent_scopes", set()):
                 return None
             else:
                 return tokens
-        return self.tokens.get(scope)
+        return self.tokens_for_environment.get(scope)
+
+    @staticmethod
+    def _deserialize_from_file(file_tokens: Dict[str, Any]) -> TokensInTokenCache:
+        deserialized: TokensInTokenCache = {}
+        for k, v in file_tokens.items():
+            if k.startswith("__"):
+                v = TokenCache._deserialize_from_file(v)
+            else:
+                v = TokenSet(**v)
+            deserialized[k] = v
+        return deserialized
 
     def load_tokens(self):
         """
@@ -95,7 +123,7 @@ class TokenCache:
         try:
             with open(self.token_store) as f:
                 contents = json.load(f)
-                self.tokens = {k: TokenSet(**v) for k, v in contents.items()}
+                self.tokens = TokenCache._deserialize_from_file(contents)
         except FileNotFoundError:
             pass
         except JSONDecodeError:
@@ -104,6 +132,17 @@ class TokenCache:
                 " and try again"
             )
 
+    @staticmethod
+    def _make_jsonable(tokens: TokensInTokenCache) -> Dict[str, Any]:
+        serialized: Dict[str, Any] = {}
+        for k, v in tokens.items():
+            if isinstance(v, TokenSet):
+                v = v._asdict()
+            elif isinstance(v, dict):
+                v = TokenCache._make_jsonable(v)
+            serialized[k] = v
+        return serialized
+
     def save_tokens(self):
         def default(x):
             if isinstance(x, set):
@@ -112,8 +151,9 @@ class TokenCache:
 
         if self.modified:
             with open(self.token_store, "w") as f:
+                jsonable = TokenCache._make_jsonable(self.tokens)
                 json.dump(
-                    {k: v._asdict() for k, v in self.tokens.items()},
+                    jsonable,
                     f,
                     indent=2,
                     sort_keys=True,
@@ -150,7 +190,7 @@ def _get_globus_sdk_native_client(
 
 def safeprint(s):
     try:
-        print(s)
+        typer.secho(s)
         sys.stdout.flush()
     except IOError:
         pass
@@ -168,13 +208,15 @@ def _do_login_for_scopes(
         refresh_tokens=True,
         prefill_named_grant=label,
     )
-    linkprompt = "Please log into Globus here"
-    safeprint(
-        "{0}:\n{1}\n{2}\n{1}\n".format(
-            linkprompt, "-" * len(linkprompt), native_client.oauth2_get_authorize_url()
-        )
+
+    linkprompt = (
+        "Please log into Globus here:\n"
+        "----------------------------\n"
+        f"{native_client.oauth2_get_authorize_url()}\n"
+        "----------------------------\n"
     )
-    auth_code = click.prompt("Enter the resulting Authorization Code here").strip()
+    safeprint(linkprompt)
+    auth_code = typer.prompt("Enter the resulting Authorization Code here")
     return native_client.oauth2_exchange_code_for_tokens(auth_code)
 
 
