@@ -1,4 +1,5 @@
 import json
+import os
 from pathlib import Path
 from typing import (
     Any,
@@ -27,6 +28,17 @@ from globus_automate_client import ActionClient
 
 PROD_FLOWS_BASE_URL = "https://flows.globus.org"
 
+_ENVIRONMENT_FLOWS_BASE_URLS = {
+    None: PROD_FLOWS_BASE_URL,
+    "prod": PROD_FLOWS_BASE_URL,
+    "production": PROD_FLOWS_BASE_URL,
+    "sandbox": "https://sandbox.flows.automate.globus.org",
+    "integration": "https://integration.flows.automate.globus.org",
+    "test": "https://test.flows.automate.globus.org",
+    "preview": "https://preview.flows.automate.globus.org",
+    "staging": "https://staging.flows.automate.globus.org",
+}
+
 MANAGE_FLOWS_SCOPE = (
     "https://auth.globus.org/scopes/eec9b274-0c81-4334-bdc2-54e90e689b9a/manage_flows"
 )
@@ -155,6 +167,13 @@ def validate_flow_definition(flow_definition: Mapping[str, Any]) -> None:
     return
 
 
+def _get_flows_base_url_for_environment():
+    environ = os.environ.get("GLOBUS_SDK_ENVIRONMENT")
+    if environ not in _ENVIRONMENT_FLOWS_BASE_URLS:
+        raise ValueError(f"Unknown value for GLOBUS_SDK_ENVIRONMENT: {environ}")
+    return _ENVIRONMENT_FLOWS_BASE_URLS[environ]
+
+
 class FlowsClient(BaseClient):
     """
     This is a specialized type of the Globus Auth service's ``BaseClient`` used
@@ -382,7 +401,7 @@ class FlowsClient(BaseClient):
             params["per_page"] = str(per_page)
         if filters is not None:
             params.update(filters)
-        if orderings is not None:
+        if orderings:
             builder = []
             for field, value in orderings.items():
                 builder.append(f"{field} {value}")
@@ -426,7 +445,13 @@ class FlowsClient(BaseClient):
         )
 
     def run_flow(
-        self, flow_id: str, flow_scope: Optional[str], flow_input: Mapping, **kwargs
+        self,
+        flow_id: str,
+        flow_scope: Optional[str],
+        flow_input: Mapping,
+        manage_by: Optional[List[str]] = None,
+        monitor_by: Optional[List[str]] = None,
+        **kwargs,
     ) -> GlobusHTTPResponse:
         """
         Run an instance of a deployed Flow with the given input.
@@ -440,6 +465,16 @@ class FlowsClient(BaseClient):
         :param flow_input: A Flow-specific dictionary specifying the input
             required for the Flow to run.
 
+        :param manage_by: A series of Globus identities which may alter
+            this Flow instance's execution. The principal value is the user's or
+            group's UUID prefixed with either 'urn:globus:groups:id:' or
+            'urn:globus:auth:identity:'
+
+        :param monitor_by: A series of Globus identities which may view this
+            Flow instance's execution state. The principal value is the user's
+            or group's UUID prefixed with either 'urn:globus:groups:id:' or
+            'urn:globus:auth:identity:'
+
         :param kwargs: Any additional kwargs passed into this method are passed
             onto the Globus BaseClient. If there exists an "authorizer" keyword
             argument, that gets used to run the Flow operation. Otherwise the
@@ -448,7 +483,7 @@ class FlowsClient(BaseClient):
         authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
         flow_url = f"{self.base_url}/flows/{flow_id}"
         ac = ActionClient.new_client(flow_url, authorizer)
-        return ac.run(flow_input, **kwargs)
+        return ac.run(flow_input, manage_by=manage_by, monitor_by=monitor_by, **kwargs)
 
     def flow_action_status(
         self, flow_id: str, flow_scope: Optional[str], flow_action_id: str, **kwargs
@@ -551,8 +586,8 @@ class FlowsClient(BaseClient):
     def list_flow_actions(
         self,
         flow_id: str,
-        flow_scope: Optional[str],
-        statuses: Optional[List[str]],
+        flow_scope: Optional[str] = None,
+        statuses: Optional[List[str]] = None,
         roles: Optional[List[str]] = None,
         marker: Optional[str] = None,
         per_page: Optional[int] = None,
@@ -610,8 +645,6 @@ class FlowsClient(BaseClient):
             argument, that gets used to run the Flow operation. Otherwise the
             authorizer_callback defined for the FlowsClient will be used.
         """
-        self.authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
-
         params = {}
         if statuses is not None and len(statuses) > 0:
             params.update(dict(filter_status=",".join(statuses)))
@@ -629,7 +662,10 @@ class FlowsClient(BaseClient):
                 builder.append(f"{field} {value}")
             params["orderby"] = ",".join(builder)
 
-        return self.get(f"/flows/{flow_id}/actions", params=params, **kwargs)
+        self.authorizer = self._get_authorizer_for_flow(flow_id, flow_scope, kwargs)
+        response = self.get(f"/flows/{flow_id}/actions", params=params, **kwargs)
+        self.authorizer = self.flow_management_authorizer
+        return response
 
     def flow_action_log(
         self,
@@ -683,7 +719,7 @@ class FlowsClient(BaseClient):
         client_id: str,
         authorizer_callback: AuthorizerCallbackType,
         authorizer: AllowedAuthorizersType,
-        base_url: str = PROD_FLOWS_BASE_URL,
+        base_url: Optional[str] = None,
         http_timeout: int = 10,
     ) -> _FlowsClient:
         """
@@ -723,6 +759,8 @@ class FlowsClient(BaseClient):
             >>> fc = FlowsClient.new_client(client_id, cli_authorizer_callback, auth)
             >>> print(fc.list_flows())
         """
+        if base_url is None:
+            base_url = _get_flows_base_url_for_environment()
         return cls(
             client_id,
             authorizer_callback,
